#!/usr/bin/python
# -*- coding: utf-8 -*-
"""層間変形角が最大になる地震力の作用角度

地震力の作用角度を-45°～45°で指定角度ずつ変更しながら繰り返し解析を行い、
各階の層間変形角が最大になる作用角度、加力方向を求めます。

"""


import csv
import os
import re
import time
import shutil
from typing import Any

from Python import Ss7Python as SS7


def import_csv(file_path: str) -> list:
    """CSVの読み込み

    CSVの情報を取得します。
    Args:
        file_path (str): CSVファイルのパス
    Returns:
        list: CSVファイルの中身のリスト

    """

    data = []
    # CSVファイルを読み込む
    with open(file_path, mode="r", encoding="cp932") as file:
        reader = csv.reader(file)
        for row in reader:
            data.append(row)

    return data


def export_csv(file_path: str, data: list) -> None:
    """CSVの出力

    パスと出力する中身を指定しCSVを出力します。
    Args:
        file_path (str): CSVファイルのパス
        data (list): CSVファイルの中身のリスト

    """

    with open(file_path, mode="w", newline="", encoding="cp932") as file:
        writer = csv.writer(file)
        writer.writerows(data)
        file.close()


def search_csv(data: list, target: str) -> int:
    """文字列検索

    listの中から指定した文字列の行番号を取得できます。
    Args:
        data (list): 文字列のリスト
        target (str): 検索したい文字列
    Returns:
        int: 行番号

    """

    line_no = []  # 行番号
    line_no = [i for i, line_s in enumerate(data) if target in line_s]
    return line_no[0]


def correction_csv(file_path: str, angle: str) -> None:
    """CSVの編集

    CSVの情報の書き換えを行ないます。
    最大ステップ数の書き換えを行って繰り返し解析を行ないます。
    Args:
        file_path (str): CSVファイルのパス
        angle (str): 地震力の作用角度

    """

    # CSVデータを格納するリスト
    data = import_csv(file_path)

    line_no = search_csv(data, "地震力の作用角度θ")
    data[line_no][4] = angle  # 最大ステップの記載場所

    # 変更したデータを新しいCSVファイルに書き出す
    export_csv(file_path, data)


def export_inp_csv(data: Any, work_path: str) -> None:
    """入力CSVのエクスポート

    SS7で入力CSVのエクスポートを行ないます。
    Args:
        data: SS7のデータ
        work_path (str): 繰り返し解析を行なう物件データのフォルダパス

    """

    # 入力データを取得する
    inp = data.GetInputData()

    # 入力CSVをエクスポートする
    # input_path = os.path.abspath(work_path +  r"\inp.csv")
    input_path = folder_path + r"\inp.csv"
    inp.ExportInputCsv(input_path, 1, 1)
    err = SS7.GetLastError()  # データを閉じる

    # 入力CSVのエクスポートが正しく実行できたことの判定
    if err.IsOK() is False:
        print("入力CSVのエクスポートが失敗しました")


def export_res_csv(data: Any, result: str, work_path: str, target: str) -> None:
    """結果CSVのエクスポート

    SS7で結果CSVのエクスポートを行ないます。
    Args:
        data: SS7のデータ
        result (str): 結果No
        work_path (str): 繰り返し解析を行なう物件データのフォルダパス
        target (str): 出力するシート名

    """

    # 計算結果データを取得する
    res = data.GetResultData(result)

    # 結果CSVをエクスポートする
    #    result_path = os.path.abspath(work_path + "\\result.csv")
    result_path = work_path + "\\result.csv"
    res.ExportResultCsv(target, result_path, 1, 1, 1)
    # 2=中断する 1=省略する 1=全部材出力
    err = SS7.GetLastError()  # データを閉じる

    # 結果CSVのエクスポートが正しく実行できたことの判定
    if err.IsOK() is False:
        print("結果CSVのエクスポートが失敗しました")


def get_csv_data(file_path: str, target: str, row: int, col: int) -> str:
    """CSVから値の取得

    CSVから指定した値を取得します。
    検索文字列からセルを検索し, その場所から指定した
    row, colだけ移動したセルから値を取得します。
    Args:
        file_path (str): CSVファイルのパス
        target (str): 検索文字列
        row: 行数
        col: 列数
    Returns:
        str: 指定セルの値

    """

    data = import_csv(file_path)
    line_no = search_csv(data, target)
    return data[line_no + row][col]


def calc_ss7(data: Any, result: str, work_path: str, target: str) -> None:
    """計算実行

    SS7で計算を実行します。
    Args:
        data: SS7のデータ
        result (str): 結果No
        work_path (str): 繰り返し解析を行なう物件データのフォルダパス
        target (str): 出力するシート名

    """

    data.DeleteResult(result)  # 結果データがある場合削除
    data.Calculate(result, "偏心率・剛性率等")

    # エラー情報を取得する
    err = SS7.GetLastError()
    # 計算が正しく実行できたことの判定
    if err.IsOK() is True:
        # 結果CSVをエクスポートする
        export_res_csv(data, result, work_path, target)
        data.Close(1)  # 1=保存する

    else:
        print("計算が失敗しました")
        # データを閉じる
        data.Close(2)  # 2=保存しない


def output_result(version: str, work_path: str, angle: str, target: str) -> None:
    """結果出力

    SS7で計算を実行して結果を出力します。
    Args:
        version (str): SS7のバージョン
        work_path (str): 繰り返し解析を行なう物件データのフォルダパス
        angle (str): 地震力の作用角度
        target (str): 出力するシート名

    """

    SS7.Init()
    SS7.Start(version, 1)  # 1=ログファイルをクリアする
    # データを開く
    #    input_path = os.path.abspath(work_path + r"\inp.csv")
    input_path = folder_path + r"\inp.csv"
    correction_csv(input_path, angle)
    SS7.CreateDataCsv(input_path, work_path, 1)  # 1=上書きする

    # エラー情報を取得する
    err = SS7.GetLastError()
    # CSV新規作成できたことの判定
    if err.IsOK() is True:
        data = SS7.Open(work_path, 1, 3)  # 1=変換して開く 3=物件データは開かない
        # データが正しく開けたことの判定
        if data is not None:
            export_inp_csv(data, work_path)
            # そのまま計算実行
            calc_ss7(data, "結果1", work_path, target)  # 結果データがある場合削除

    SS7.End(1)  # 1＝保存して終了


def get_delta(
    Floor: int,
    angle: int,
    out_path: str,
    delta_list: list,
) -> None:
    """層間変形角の取得

    各階で最大の層間変形角をリストに格納する。
    Args:
        Floor (int): 物件階数
        angle (int): 地震力の作用角度
        out_path (str): 出力パス
        delta_list (list): 結果リスト

    """

    for i in range(Floor):
        temp = get_csv_data(out_path, "case=EX+", 4 + i, 8)
        DeltaX1 = int(re.findall("[0-9]+$", temp)[0])  # 層間変形角の分母を取得
        temp = get_csv_data(out_path, "case=EX-", 4 + i, 8)
        DeltaX2 = int(re.findall("[0-9]+$", temp)[0])
        temp = get_csv_data(out_path, "case=EY+", 4 + i, 8)
        DeltaY1 = int(re.findall("[0-9]+$", temp)[0])
        temp = get_csv_data(out_path, "case=EY-", 4 + i, 8)
        DeltaY2 = int(re.findall("[0-9]+$", temp)[0])
        # EX+時
        if "0" == delta_list[1 + i][10] or DeltaX1 < int(
            re.findall("[0-9]+$", delta_list[1 + i][8])[0]
        ):
            delta_list[1 + i][0] = get_csv_data(out_path, "case=EX+", 4 + i, 0)  # 階
            delta_list[1 + i][1] = get_csv_data(out_path, "case=EX+", 4 + i, 1)  # X軸
            delta_list[1 + i][2] = get_csv_data(out_path, "case=EX+", 4 + i, 2)  # Y軸
            delta_list[1 + i][3] = get_csv_data(out_path, "case=EX+", 4 + i, 3)  # 構造
            delta_list[1 + i][4] = get_csv_data(out_path, "case=EX+", 4 + i, 4)  # 階高
            delta_list[1 + i][5] = get_csv_data(out_path, "case=EX+", 4 + i, 5)  # δX
            delta_list[1 + i][6] = get_csv_data(out_path, "case=EX+", 4 + i, 6)  # δY
            delta_list[1 + i][7] = get_csv_data(out_path, "case=EX+", 4 + i, 7)  # δ
            delta_list[1 + i][8] = "1/" + str(DeltaX1)
            delta_list[1 + i][9] = str(angle)
            delta_list[1 + i][10] = "EX+"
        # EX-時
        if DeltaX2 < int(re.findall("[0-9]+$", delta_list[1 + i][8])[0]):
            delta_list[1 + i][0] = get_csv_data(out_path, "case=EX-", 4 + i, 0)  # 階
            delta_list[1 + i][1] = get_csv_data(out_path, "case=EX-", 4 + i, 1)  # X軸
            delta_list[1 + i][2] = get_csv_data(out_path, "case=EX-", 4 + i, 2)  # Y軸
            delta_list[1 + i][3] = get_csv_data(out_path, "case=EX-", 4 + i, 3)  # 構造
            delta_list[1 + i][4] = get_csv_data(out_path, "case=EX-", 4 + i, 4)  # 階高
            delta_list[1 + i][5] = get_csv_data(out_path, "case=EX-", 4 + i, 5)  # δX
            delta_list[1 + i][6] = get_csv_data(out_path, "case=EX-", 4 + i, 6)  # δY
            delta_list[1 + i][7] = get_csv_data(out_path, "case=EX-", 4 + i, 7)  # δ
            delta_list[1 + i][8] = "1/" + str(DeltaX2)
            delta_list[1 + i][9] = str(angle)
            delta_list[1 + i][10] = "EX-"
        # EY+時
        if DeltaY1 < int(re.findall("[0-9]+$", delta_list[1 + i][8])[0]):
            delta_list[1 + i][0] = get_csv_data(out_path, "case=EY+", 4 + i, 0)  # 階
            delta_list[1 + i][1] = get_csv_data(out_path, "case=EY+", 4 + i, 1)  # X軸
            delta_list[1 + i][2] = get_csv_data(out_path, "case=EY+", 4 + i, 2)  # Y軸
            delta_list[1 + i][3] = get_csv_data(out_path, "case=EY+", 4 + i, 3)  # 構造
            delta_list[1 + i][4] = get_csv_data(out_path, "case=EY+", 4 + i, 4)  # 階高
            delta_list[1 + i][5] = get_csv_data(out_path, "case=EY+", 4 + i, 5)  # δX
            delta_list[1 + i][6] = get_csv_data(out_path, "case=EY+", 4 + i, 6)  # δY
            delta_list[1 + i][7] = get_csv_data(out_path, "case=EY+", 4 + i, 7)  # δ
            delta_list[1 + i][8] = "1/" + str(DeltaY1)
            delta_list[1 + i][9] = str(angle)
            delta_list[1 + i][10] = "EY+"
        # EY-時
        if DeltaY2 < int(re.findall("[0-9]+$", delta_list[1 + i][8])[0]):
            delta_list[1 + i][0] = get_csv_data(out_path, "case=EY-", 4 + i, 0)  # 階
            delta_list[1 + i][1] = get_csv_data(out_path, "case=EY-", 4 + i, 1)  # X軸
            delta_list[1 + i][2] = get_csv_data(out_path, "case=EY-", 4 + i, 2)  # Y軸
            delta_list[1 + i][3] = get_csv_data(out_path, "case=EY-", 4 + i, 3)  # 構造
            delta_list[1 + i][4] = get_csv_data(out_path, "case=EY-", 4 + i, 4)  # 階高
            delta_list[1 + i][5] = get_csv_data(out_path, "case=EY-", 4 + i, 5)  # δX
            delta_list[1 + i][6] = get_csv_data(out_path, "case=EY-", 4 + i, 6)  # δY
            delta_list[1 + i][7] = get_csv_data(out_path, "case=EY-", 4 + i, 7)  # δ
            delta_list[1 + i][8] = "1/" + str(DeltaY2)
            delta_list[1 + i][9] = str(angle)
            delta_list[1 + i][10] = "EY-"


def loop(version: str, work_path: str) -> None:
    """繰り返し解析

    指定ステップ毎に繰り返し解析を行ないます。
    Args:
        version (str): SS7のバージョン
        work_path (str): 繰り返し解析を行なう物件データのフォルダパス

    """

    input_path = folder_path + r"\inp.csv"
    out_path = work_path + "\\result.csv"

    Floor = int(get_csv_data(input_path, "全階数", 0, 3))
    Under = int(get_csv_data(input_path, "地下階数", 0, 3))
    Floor -= Under  # 地下階は除く

    result = [[str(0)] * 11 for i in range(Floor + 1)]  # 結果格納
    result[0] = [
        "階",
        "X軸",
        "Y軸",
        "柱構造",
        "階高",
        "δx",
        "δy",
        "δ",
        "最大層間変形角",
        "角度",
        "加力方向",
    ]  # ヘッダーを設定

    print("-45°~45°まで" + angle + "°ずつ解析を行なう")  # 動作概要

    for i in range(-45, 45, int(angle)):  # 開始, 終了, 区切り
        print(str(i) + "°")  # 途中経過表示
        output_result(version, work_path, str(i), "10.13.層間変形角")
        get_delta(Floor, i, out_path, result)

    # CSVエクスポート
    export_csv(folder_path + "_result.csv", result)


def main() -> None:
    """メイン処理

    最初に動く関数です。

    """

    start = time.time()  # 処理時間計測開始
    SS7.Init()
    SS7.Start(version, 1)  # 1=ログファイルをクリアする
    # データを開く
    data = SS7.Open(folder_path, 1, 3)  # 1=変換して開く 3=物件データは開かない
    # データが正しく開けたことの判定
    if data is not None:
        # 繰り返し更新を行なう一時的な物件を作成
        work_path = os.path.abspath(folder_path + r"\..\Temp.ikn")
        export_inp_csv(data, work_path)

        SS7.End(1)  # 1=保存して終了

        loop(version, work_path)    # 繰り返し解析
        shutil.rmtree(work_path)    # Tempフォルダの削除

    else:
        SS7.End(1)  # 1=保存して終了
        print("物件を開けませんでした")
        return

    end = time.time()  # 処理時間計測終了
    count_time = format(end - start, '.1f')
    print(str(count_time) + "秒かかりました")


if __name__ == "__main__":
    # 条件の指定はここで行なってください
    version = "1.1.1.20"  # SS7のバージョン

    folder_path = r"C:\UsrData\Ss7Data\TEST.ikn"  # 物件フォルダパス
    angle = "5"  # 地震力の区切り角度

    main()
